package CheckRelease2::CMS::ContentData;

use strict;

use MT::CMS::ContentData;
use MT::ContentType;
use MT::Util;
use JSON;

sub save_rev {
  my ($app) = @_;
  my $blog  = $app->blog;
  my $cfg   = $app->config;
  my $param = {};

  # Validate magic token
  $app->validate_magic
      or return $app->errtrans("Invalid request.");

  # Parameter check
  my $blog_id = $app->param('blog_id')
      or return $app->errtrans("Invalid request.");
  my $content_type_id = $app->param('content_type_id')
      or return $app->errtrans("Invalid request.");
  my $content_type = MT::ContentType->load($content_type_id)
      or return $app->errtrans("Invalid request.");
  my $field_data          = $content_type->fields;
  my $content_field_types = $app->registry('content_field_types');

  # Permission check
  my $perms = $app->permissions
      or return $app->permission_denied();

  my $content_data_id = $app->param('id');
  if ( !$content_data_id ) {
      return $app->permission_denied()
          unless $perms->can_do('create_new_content_data')
          || $perms->can_do(
          'create_new_content_data_' . $content_type->unique_id );
  }
  else {
      return $app->permission_denied()
          unless $perms->can_edit_content_data( $content_data_id,
          $app->user );
  }

  my $convert_breaks = {};
  my $data           = {};
  if ( $app->param('from_preview') ) {
      $data = JSON::decode_json( scalar $app->param('serialized_data') );
  }
  else {
      foreach my $f (@$field_data) {
          my $content_field_type = $content_field_types->{ $f->{type} };
          $data->{ $f->{id} }
              = MT::CMS::ContentData::_get_form_data( $app, $content_field_type, $f );
          if ( $f->{type} eq 'multi_line_text' ) {
              $convert_breaks->{ $f->{id} } = $app->param(
                  'content-field-' . $f->{id} . '_convert_breaks' );
              my $key = $f->{id} . '_convert_breaks';
              $data->{$key}
                  = $app->param(
                  'content-field-' . $f->{id} . '_convert_breaks' );
          }
      }
  }

  if ( $app->param('_autosave') ) {
      return MT::CMS::ContentType::_autosave_content_data( $app, $data );
  }

  if ( my $errors = MT::CMS::ContentData::_validate_content_fields( $app, $content_type, $data ) )
  {
      $app->param( '_type',           'content_data' );
      $app->param( 'reedit',          1 );
      $app->param( 'serialized_data', $data );
      my %param;
      $param{err_msg} = $errors->[0]{error};
      return $app->forward( 'view_content_data', \%param );
  }

  my $content_data
      = $content_data_id
      ? MT::ContentData->load($content_data_id)
      : MT::ContentData->new();

  my $archive_type = '';

  my $orig      = $content_data->clone;
  my $orig_file = '';
  if ( $content_data->id ) {
      $archive_type = 'ContentType';
      $orig_file
          = MT::Util::archive_file_for( $orig, $blog, $archive_type );
  }
  my $status_old = $content_data_id ? $content_data->status : 0;

  if ( $content_data->id ) {
      $content_data->modified_by( $app->user->id );

      $archive_type = 'ContentType';
      $orig_file
          = MT::Util::archive_file_for( $orig, $blog, $archive_type );
  }
  else {
      $content_data->author_id( $app->user->id );
      $content_data->blog_id($blog_id);
  }
  $content_data->content_type_id($content_type_id);
  $content_data->data($data);

  $content_data->identifier( scalar $app->param('identifier') );

  if ( $app->param('scheduled') ) {
      $content_data->status( MT::ContentStatus::FUTURE() );
  }
  else {
      my $status = $app->param('status');
      $content_data->status($status);
  }
  if ( ( $content_data->status || 0 ) != MT::ContentStatus::HOLD() ) {
      if ( !$blog->site_path || !$blog->site_url ) {
          return $app->error(
              $app->translate(
                  "Your blog has not been configured with a site path and URL. You cannot publish entries until these are defined."
              )
          );
      }
  }

  my $ao_d = $app->param('authored_on_date');
  my $ao_t = $app->param('authored_on_time');
  my $uo_d = $app->param('unpublished_on_date');
  my $uo_t = $app->param('unpublished_on_time');

  my ( $previous_old, $next_old );

  # TODO: permission check
  if ($ao_d) {
      my %param = ();
      my $ao    = $ao_d . ' ' . $ao_t;
      my $ts    = MT::Util::valid_date_time2ts($ao);
      if ( !$ts ) {
          $param{err_msg} = $app->translate(
              "Invalid date '[_1]'; 'Published on' dates must be in the format YYYY-MM-DD HH:MM:SS.",
              $ao
          );
      }
      $param{return_args} = $app->param('return_args');
      if ( $param{err_msg} ) {
          $app->param( '_type',           'content_data' );
          $app->param( 'reedit',          1 );
          $app->param( 'serialized_data', $data );
          return $app->forward( "view_content_data", \%param );
      }
      if ( $content_data->authored_on ) {
          $previous_old = $content_data->previous(1);
          $next_old     = $content_data->next(1);
      }
      $content_data->authored_on($ts);
  }

  # TODO: permission check
  if ( $content_data->status != MT::ContentStatus::UNPUBLISH() ) {
      if ( $uo_d || $uo_t ) {
          my %param = ();
          my $uo    = $uo_d . ' ' . $uo_t;
          my $ts    = MT::Util::valid_date_time2ts($uo);
          if ( !$ts ) {
              $param{err_msg} = $app->translate(
                  "Invalid date '[_1]'; 'Unpublished on' dates must be in the format YYYY-MM-DD HH:MM:SS.",
                  $uo
              );
          }
          unless ( $param{err_msg} ) {
              $param{err_msg} = $app->translate(
                  "Invalid date '[_1]'; 'Unpublished on' dates should be dates in the future.",
                  $uo
                  )
                  if (
                  MT::DateTime->compare(
                      blog => $blog,
                      a    => { value => time(), type => 'epoch' },
                      b    => $ts
                  ) > 0
                  );
          }
          if ( !$param{err_msg} && $content_data->authored_on ) {
              $param{err_msg} = $app->translate(
                  "Invalid date '[_1]'; 'Unpublished on' dates should be later than the corresponding 'Published on' date.",
                  $uo
                  )
                  if (
                  MT::DateTime->compare(
                      blog => $blog,
                      a    => $content_data->authored_on,
                      b    => $ts
                  ) > 0
                  );
          }
          $param{show_input_unpublished_on} = 1 if $param{err_msg};
          $param{return_args} = $app->param('return_args');
          if ( $param{err_msg} ) {
              $app->param( '_type',           'content_data' );
              $app->param( 'reedit',          1 );
              $app->param( 'serialized_data', $data );
              return $app->forward( "view_content_data", \%param );
          }
          if ( $content_data->unpublished_on ) {
              $previous_old = $content_data->previous(1);
              $next_old     = $content_data->next(1);
          }
          $content_data->unpublished_on($ts);
      }
      else {
          $content_data->unpublished_on(undef);
      }
  }

  my $is_new = $content_data->id ? 0 : 1;

  $content_data->convert_breaks(
      MT::Serialize->serialize( \$convert_breaks ) );

  if ( !$content_type->data_label ) {
      my $data_label = $app->param('data_label');
      $content_data->label($data_label);
  }

  $app->run_callbacks( 'cms_pre_save.content_data',
      $app, $content_data, $orig );
}
1;