package CheckRelease2::Plugin;

use strict;
use warnings;

use MT;
use base qw(MT::Plugin);

use vars qw( $VERSION $OLD_VERSION $SCHEMA_VERSION );
$VERSION = '2.1.02';
$OLD_VERSION = 0;
$SCHEMA_VERSION = 0.025;

use CheckRelease2::L10N;
use CheckRelease2::Util qw(license_manager cond_can_manage_workflow enable_plugin);
use CherryPick::LicenseManager;
use CherryPick::LicenseManager::CMS;
use CheckRelease2::CMS::Config;
use CheckRelease2::Task;

my $plugin = CheckRelease2::Plugin->new({
  id => 'CheckRelease2',
  key => __PACKAGE__,
  name => 'CheckRelease',
  l10n_class => 'CheckRelease2::L10N',
  description => '<__trans phrase="Support your checking entries." />',
  doc_link => 'https://www.cherry-pick.jp/mt-plugins/checkrelease/',
  author_name => "cherry-pick Inc.",
  author_link => 'https://www.cherry-pick.jp/mt-plugins/checkrelease/',
  system_config_template => \&CheckRelease2::CMS::Config::system_config,
  blog_config_template => \&CheckRelease2::CMS::Config::blog_config,
  settings => new MT::PluginSettings([
    ['lock_publish'],
  ]),
  schema_version => $SCHEMA_VERSION,
  version => $VERSION,
});

sub init_registry {
  my ($plugin) = @_;
  my $cfg = MT->config;
  my $pkg = '$CheckRelease2::CheckRelease2::';
  $plugin->registry({
    config_settings => {
      'CRPreviewURLExpires' => { default => 60 * 60 * 24 * 14 },
      'CRRedirectURLExpires' => { default => 60 * 5 },
    },
    object_types => {
      cr2_approval => 'CheckRelease2::Approval',
      cr2_step => 'CheckRelease2::Step',
      cr2_comment => 'CheckRelease2::Comment',
      cr2_rule => 'CheckRelease2::Rule',
    },
    listing_screens => {
      cr2_step => {
        object_label => 'Workflow',
        object_label_plural => 'Workflow',
        primary      => 'name',
        view         => [qw( website blog )],
        default_sort_by => 'name',
        condition => sub {
          cond_can_manage_workflow();
        },
        data_api_condition => sub {0},
      },
    },
    enable_object_methods => {
      cr2_step => {
        delete => 1,
        edit => $pkg .'CMS::Step::view',
        save => 1,
      },
    },
    default_templates => {
      base_path => 'tmpl/default_templates',
      'global:email' => {
        remand_email => {
          label => 'Return to notification',
        },
        approval_request_email => {
          label => 'Approval request notification',
        },
        approved_email => {
          label => 'Approved notification',
        },
      },
    },
    callbacks => {
      'MT::App::CMS::init_request' => {
        priority => 1,
        code => $pkg .'CMS::on_init_request',
      },
    },
    applications => {
      cms => {
        methods => {
          cr2_register_license_manager => sub {
            CherryPick::LicenseManager::CMS::handle_register_licensekey(license_manager, @_);
          },
          save_cr2_step => $pkg .'CMS::Step::save',
          rules => $pkg .'CMS::Step::edit_rules',
          save_cr2_rules => $pkg .'CMS::Step::save_rules',
          js_author_list => $pkg .'CMS::Author::js_author_list',
          list_cr2_comment => $pkg .'CMS::Comment::list',
          view_rule => $pkg .'CMS::Rule::view',
          save_entry_rev => $pkg .'CMS::Entry::save_rev',
          save_cd_rev => $pkg .'CMS::ContentData::save_rev',
        },
        menus => {
          checkrelease2 => {
            label => 'Workflow',
            order => 1450,
            permission => 'manage_workflow',
          },
          'checkrelease2:list' => {
            label => 'Workflow list',
            order => 1460,
            mode => 'list',
            args => { _type => 'cr2_step' },
            permission => 'manage_workflow',
            view => ['blog', 'website'],
            condition => sub {
              enable_plugin() ? 1 : 0;
            }
          },
          'checkrelease2:view' => {
            label => 'Workflow create',
            order => 1470,
            mode => 'view_rule',
            args => { _type => 'cr2_step' },
            permission => 'manage_workflow',
            view => ['blog', 'website'],,
            condition => sub {
              enable_plugin() ? 1 : 0;
            }
          },
          'checkrelease2:rules' => {
            label => 'Workflow rules',
            order => 1480,
            mode => 'rules',
            args => { _type => 'cr2_step' },
            permission => 'manage_workflow',
            view => ['blog', 'website'],,
            condition => sub {
              enable_plugin() ? 1 : 0;
            }
          },
        },
        callbacks => {
          'template_source.header' => [
            {
              code => $pkg .'CMS::Step::_hdlr_template_source_header',
              priority => 1,
            },
            {
              code => $pkg .'CMS::Entry::_hdlr_template_source_header',
              priority => 1,
            },
          ],
          'template_param.edit_role' => $pkg .'CMS::Role::_hdlr_template_param_edit_role',
          'template_param.cfg_plugin' => sub {
            CherryPick::LicenseManager::CMS::handle_template_param_cfg_plugin(license_manager, @_);
          },
          'template_param.edit_entry' => $pkg .'CMS::Entry::_hdlr_template_param_edit_entry',
          'template_source.edit_entry' => $pkg .'CMS::Entry::_hdlr_template_source_edit_entry',
          'template_param.edit_content_data' => $pkg .'CMS::Entry::_hdlr_template_param_edit_entry',
          'template_source.edit_content_data' => $pkg .'CMS::Entry::_hdlr_template_source_edit_entry',

          'cms_pre_save.entry' => $pkg .'CMS::Entry::_hdlr_cms_pre_save_entry_rev',
          'cms_pre_save.page' => $pkg .'CMS::Entry::_hdlr_cms_pre_save_entry_rev',
          'cms_pre_save.content_data' => $pkg .'CMS::Entry::_hdlr_cms_pre_save_entry_rev',
          'cms_post_save.entry' => $pkg .'CMS::Entry::_hdlr_cms_post_save_entry',
          'cms_post_save.page' => $pkg .'CMS::Entry::_hdlr_cms_post_save_entry',
          'cms_post_save.content_data' => $pkg .'CMS::Entry::_hdlr_cms_post_save_entry',
          'cms_view_permission_filter.cr2_step' => $pkg .'CMS::Step::_hdlr_cms_view_permission_filter',
          'MT::App::CMS::app_pre_listing_list_revision' => $pkg .'CMS::Entry::_hdlr_app_pre_listing_list_revision',

          'cms_filtered_list_param.entry' => $pkg .'CMS::Entry::_hdlr_cms_filtered_list_param',
          'cms_filtered_list_param.page' => $pkg .'CMS::Entry::_hdlr_cms_filtered_list_param',
        },
      },
    },
    list_properties => {
      entry => {
        cr_approval_status => {
          base => '__common.string',
          label => sub { MT->component('CheckRelease2')->translate('Approval Status'); },
          sort => 0,
          html => $pkg .'ListProperty::_hdlr_entry_approval_status_html',
          condition => $pkg .'ListProperty::_hdlr_condition',
          order => 110,
          display => 'default',
        },
        cr_revision => {
          base => '__common.string',
          label => sub { MT->component('CheckRelease2')->translate('Revision'); },
          sort => 0,
          html => $pkg .'ListProperty::_hdlr_entry_revision_html',
          condition => $pkg .'ListProperty::_hdlr_condition',
          order => 210,
          display => 'default',
        },
      },
      page => {
        cr_approval_status => {
          base => '__common.string',
          label => sub { MT->component('CheckRelease2')->translate('Approval Status'); },
          sort => 0,
          html => $pkg .'ListProperty::_hdlr_entry_approval_status_html',
          condition => $pkg .'ListProperty::_hdlr_condition',
          order => 110,
          display => 'default',
        },
        cr_revision => {
          base => '__common.string',
          label => sub { MT->component('CheckRelease2')->translate('Revision'); },
          sort => 0,
          html => $pkg .'ListProperty::_hdlr_entry_revision_html',
          condition => $pkg .'ListProperty::_hdlr_condition',
          order => 210,
          display => 'default',
        },
      },
      cr2_step => $pkg .'Step::list_prop',
    },
    tags => {
      block => {
        'HasCR2Comments?' => $pkg .'Template::Tags::_hdlr_has_cr2_comments',
        CR2Comments => $pkg .'Template::Tags::_hdlr_comments',
        'HasCR2StatusChanged?' => $pkg .'Template::Tags::_hdlr_has_cr2_status_changed',
      },
      function => {
        CR2CommentDate => $pkg .'Template::Tags::_hdlr_comment_date',
        CR2CommentText => $pkg .'Template::Tags::_hdlr_comment_text',
        CR2ApprovalStatusLabel => $pkg .'Template::Tags::_hdlr_approval_status_label',
        CR2CommentStatus => $pkg .'Template::Tags::_hdlr_comment_status',
        CR2CommentStatusLabel => $pkg .'Template::Tags::_hdlr_comment_status_label',
      }
    },
    permissions => {
      'blog.manage_workflow' => {
        group => 'workflow',
        label => 'Edit workflow',
        order => '2010',
      },
      'blog.intermediate_approver' => {
        group => 'workflow',
        label => 'Intermediate approver',
        order => '2011',
      },
      'blog.final_approver' => {
        group => 'workflow',
        label => 'Final approver',
        order => '2012',
      },
      'blog.administer_site' => {
        inherit_from => [
          'blog.manage_worklow',
          'blog.intermediate_approver',
          'blog.final_approver',
        ],
      },
    },
    tasks => {
      'RevisionFuturePost' => {
        key => 'RevisionFuturePost',
        label => "Publish Scheduled Content's Revision",
        frequency => $cfg->FuturePostFrequency,
        code => sub {
          CheckRelease2::Task->future_post;
        },
      },
    },
  });
}

sub instance {
  $plugin;
}

1;
