package CheckRelease2::Util;

use strict;

use base 'Exporter';
our @EXPORT_OK = qw(
  _dump
  _log
  _debug
  _dt
  plugin
  _t
  license_manager
  unable_plugin
  enable_plugin
  trim
  array_search
  get_plugin_config
  do_redirect
  get_users_by_perms
  lookup_rule
  lookup_approval
  lookup_step
  can_final_approval
  can_intermediate_approval
  can_manage_workflow
  cond_can_manage_workflow
  can_create
  send_workflow_emails
  set_param_session
  get_param_session
  sanitize
  cf_meta
  check_url

  get_config_value
  load_latest_revision
  register_future_que
  serialize
  unserialize
  set_meta
  now
  gather_cb_beacon
  is_mtcloud
  add_alt_tmpl_path_mtcloud
  has_lookup_rule
);

use MT;
use MT::Mail;
use MT::Log;
use MT::App::CMS;
use MT::Util qw(format_ts is_valid_email encode_html);
use CheckRelease2::Approval;
use CheckRelease2::ApprovalStatus qw(:all);
use Data::Dumper;

sub plugin {
  MT->component('CheckRelease2');
}

sub _dump {
  use Data::Dumper;
  Dumper @_;
}

sub _log {
  my ($msg, $is_blog, $meta) = @_;
  my $params = {
    category => 'info',
    level    => MT::Log::INFO(),
    message  => $msg,
  };
  $params->{class} = 'blog' if defined $is_blog;
  $params->{meta} = _dump $meta if defined $meta;
  MT->instance->log($params);
}

sub _debug {
  MT::log($_[0]);
}

sub _dt {
  my @caller = caller;
  MT::log($caller[1].'::'.$caller[2]);
}

sub _decode_utf8 {
  Encode::is_utf8($_[0]) ? $_[0] : Encode::decode_utf8($_[0]);
}

sub _encode_utf8 {
  Encode::is_utf8($_[0]) ? Encode::encode_utf8($_[0]) : $_[0];
}

sub _t {
  plugin->translate(@_);
}

sub license_manager {
  my $plugin = plugin();
  no strict;
  my $local_active_env_name = $plugin->name .'LocalActivation';
  my $lm = CherryPick::LicenseManager->new({
    plugin          => $plugin,
    plugin_key      => $plugin->id,
    register_url    => 'https://mtlmc.cherry-pick.jp/api/licenses/register',
    confirm_url     => 'https://mtlmc.cherry-pick.jp/api/licenses/register',
    confirm_interval => 14,
    trial_days => 30,
    local_activation => MT->config->$local_active_env_name ? 1 : 0,
  }) || return $plugin->error(CherryPick::LicenseManager->errstr);
  $lm;
}

sub unable_plugin {
  !enable_plugin();
}

sub enable_plugin {
  my $lm = license_manager();
  return 1 if $lm->is_registered;
  return 1 unless $lm->is_expired;
  0;
}

sub trim {
  $_ = $_[0];
  return unless $_;
  s!^\s*!!;
  s!\s*$!!;
  $_;
}

sub array_search {
  my ($arr, $needle) = @_;
  return undef unless ref $arr eq 'ARRAY';
  foreach ( @$arr ) {
    return 1 if $_ eq $needle;
  }
  -1;
}

sub do_redirect {
  my ($app, $url) = @_;
  my $q = $app->param;
  my $qr = ref $q;
  if ( $ENV{MOD_PERL} ) {
      $app->{apache}->header_out( Location => $url );
      $app->response_code( Apache::Constants::REDIRECT() );
      $app->send_http_header;
  }
  elsif ( $qr =~ /::PSGI/i ) {
    $app->redirect($url);
  }
  else {
    $app->print(
      $q->redirect( -uri => $url, %{ $app->{cgi_headers} } ) );
  }
  1;
}

sub get_users_by_perms {
  my ($perms, $blog_id) = @_;
  return unless defined $perms;
  $blog_id = 0 unless defined($blog_id) && $blog_id =~ m!^\d+$!;
  my $terms = [];
  if ( ref($perms) eq 'ARRAY' ) {
    foreach my $perm ( @$perms ) {
      push @$terms, '-or' if @$terms;
      push @$terms, { permissions => { like => "%${perm}\'%" }, blog_id => $blog_id };
    }
  } else {
    push @$terms, { permissions => { like => "%${perms}\'%" }, blog_id => $blog_id };
  }
  my @perms = MT->model('permission')->load($terms);
  return unless @perms;
  my @ids = map { $_->author_id } @perms;
  my @authors = MT->model('author')->load({ id => \@ids });
  @authors ? \@authors : undef;
}

sub lookup_rule {
  my ($blog_id, $object_id) = @_;
  my $class = MT->instance->model('cr2_rule');
  my $terms = { blog_id => $blog_id };
  if ( $object_id eq 'entry' || $object_id eq 'page' ) {
    $terms->{object_ds} = $object_id;
  } else {
    $terms->{object_id} = $object_id;
  }
  $class->load($terms) || undef;
}

sub lookup_approval {
  my ($id, $class, $rev_number) = @_;
  return unless defined $id && defined $class;
  my $terms = {
    object_id => $id
  };
  if ( $class eq 'entry' || $class eq 'page' ) {
    $terms->{object_class} = $class;
  } else {
    $terms->{content_type_id} = $class;
    $terms->{object_class} = 'content_data';
  }
  my $approval = MT->model('cr2_approval')->load($terms);
  return unless defined $approval;
  if ( $rev_number ) {
    $approval->is_rev( 1 );
    $approval->rev_number( $rev_number );
  } else {
    $approval->is_rev( 0 );
  }
  $approval;
}

sub lookup_step {
  my ($step_id) = @_;
  return unless defined $step_id;
  MT->model('cr2_step')->load({ id => $step_id });
}

sub can_final_approval {
  my ($user, $blog_id) = @_;
  my $perms = $user->permissions($blog_id)->permissions;
  return 1 if $perms =~ m!(final_approver|administer)!;
  0;
}

sub can_intermediate_approval {
  my ($user, $blog_id) = @_;
  my $perms = $user->permissions($blog_id)->permissions;
  return 1 if $perms =~ m!(intermediate_approver|administer)!;
  0;
}

sub can_manage_workflow {
  my ($user, $blog_id) = @_;
  return 0 unless $user;
  my $perms = $user->permissions($blog_id)->permissions;
  return 1 if $perms =~ m!(administer|manage_workflow)!;
  0;
}

sub cond_can_manage_workflow {
  my $app = MT->instance;
  $app = MT::App::CMS->instance;
  my $user = $app->user;
  #  or die $app->error(
  #    $app->translate('Internal Error: Login user is not initialized.') );
  my $blog_id = $app->param('blog_id') || 0;
  can_manage_workflow($user, $blog_id);
}

sub can_create {
  my ($user, $type, $blog) = @_;
  require MT::Permission;
  my @perms = MT::Permission->load(
    { author_id => $user->id,
      ( defined $blog ? ( blog_id => $blog->id ) : ( blog_id => { not => 0 } ) )
    }
  );
  my $perm = $type eq 'entry'
    ? 'create_post'
    : $type eq 'page'
      ? 'manage_pages'
      : '';
  if ( $type =~ m!^\d+! ) {
    $perm = $perms[0];
    
    my $ct = MT->model('content_type')->load({ id => $type });
    if ( defined $ct ) {
      return 1 if $perm->can_edit_content_data($ct->id, $user);
    }
  }
  return 0 unless $perm;
  foreach my $p ( @perms ) {
    return 1 if $p->has($perm);
  }
  0;
}

sub check_url {
  my ($app, $obj, $mode) = @_;
  my $q = $app->param;
  my $plugin = plugin();
  my $args;
  my $blog = $obj->blog;

  my $class = $obj->class_type eq 'page' ? 'entry' : $obj->class_type;
  my $columns = $obj->column_names;

  if ( $mode eq 'preview' ) {
    $mode = 'preview_' .$class;
    foreach ( @$columns ) {
      if ( $_ eq 'authored_on' || $_ eq 'created_on' || $_ eq 'modified_on' || $_ eq 'unpublished_on' ) {
        $args->{$_ .'_date'} = format_ts('%Y-%m-%d', $obj->$_, $blog, 'en', 0);
        $args->{$_ .'_time'} = format_ts('%H:%M:%S', $obj->$_, $blog, 'en', 0);
      }
      else {
        $args->{$_} = $obj->$_;
      }
    }
    foreach ( qw/include_asset_ids category_ids tags/ ) {
      $args->{$_} = $q->param($_);
    }

    if ( $class eq 'entry' || $class eq 'page' ) {
      my $cf_meta = cf_meta($app, $obj);
      foreach ( keys %$cf_meta ) {
        my $cf = "customfield_$_";
        $args->{$cf} = $cf_meta->{$_};
        if ( $args->{$cf} =~ /^\d{14}$/ ) {
          $args->{"d_$cf"} = format_ts('%Y-%m-%d', $cf_meta->{$_}, $obj->blog, 'en', 0);
          $args->{"t_$cf"} = format_ts('%H:%M:%S', $cf_meta->{$_}, $obj->blog, 'en', 0);
        }
      }
      $args->{customfield_beacon} = 1;
    }
    my %params = $app->param_hash;
    foreach ( keys %params ) {
      $args->{$_} ||= $params{$_};
    }
  }
  else {
    map { $args->{$_} = $q->param($_) } grep { $q->param($_) } qw/_type blog_id content_type_id type crr/;
    if ( $args->{crr} ) {
      my $rev = load_latest_revision($app, $class eq 'content_data' ? $obj->id : $class, $obj->id);
      $args->{r} = $rev->rev_number;
    }
    $args->{id} = $obj->id;
  }

  # セッションパラメータによりURLを短縮
  my $blog_id = $blog->id;
  my ( $token, $salt ) = set_param_session( $app->config('CRPreviewURLExpires'), $args );
  $args = { cr_token => $token, cr_salt => $salt };
  $args->{blog_id} = $blog_id if defined $blog_id;

  my $uri = $app->config->CGIPath;
  $uri .= $app->config->AdminScript . $app->uri_params(mode => $mode, args => $args);
  unless ( $uri =~ /^https?/ ) {
    #[TODO] on MTCloud
    #my $config = $plugin->load_config(\%config, 'system');
    #my $mtadmin_host = defined $config->{admin_host_uri} ? $config->{admin_host_uri} : 
    my $blog_url = $blog->site_url;
    $blog_url .= '/' unless $blog_url =~ m!/$!;
    my ($site_url) = ($blog_url =~ m!^(https?://[^/]+)/!);
    $uri = "$site_url$uri";
  }

  #$uri =~ s/^http:/https:/ if $config{use_ssl_url};
  $uri;
}

sub set_param_session {
    my( $expires, $param ) = @_;

    my $t = time;
    $expires = int($expires);

    my $salt = int(rand(1000000));
    require Digest::MD5;
    my $token = Digest::MD5::md5_hex(time + $salt);

    require MT::Session;
    my $sess = MT::Session->new;
    $sess->set_values({
        kind        => '_C',
        id          => $token,
        name        => $salt,
        start       => $t,
        duration    => $t + $expires,
    });
    $sess->set('param', $param);
    $sess->save;

    ( $token, $salt );
}

sub get_param_session {
  my ( $token, $salt ) = @_;
  require MT::Session;
  my $sess = MT::Session->load({
    kind     => '_C',
    id       => $token,
    name     => $salt,
    duration => [ time, undef ],
  }, {
    range    => { duration => 1 },
  });
  return unless $sess;
  my $param = $sess->get('param');
  $param;
}

sub send_workflow_emails {
  my ($author, $blog, $class, $obj, $status, $steps, $current, $approval, $comment) = @_;
  my $app = MT->instance;

  # load template each status and each role
  return if !$status && !$comment;
  my ($id, $header, $body);
  $header->{From} = $author->email;
  $header->{'Reply-To'} = $author->email;
  if ( $status == CHECKING || !$status) {
    $id = 'approval_request_email';
    $header->{Subject} = $status
      ? _t(
        'Approval application was requested on [_1](ID: [_2]) by [_3](ID: [_4])',
        $blog->name, $blog->id, $author->nickname, $author->id
        )
      : _t(
        'Comment was posted on [_1](ID: [_2]) by [_3](ID: [_4])',
        $blog->name, $blog->id, $author->nickname, $author->id
        );
  } elsif ( $status == REMAND || $status == DISMISSAL ) {
    # TODO: in case of content_type, what field 
    $id = 'remand_email';
    $header->{Subject} = _t(
      'The application for approval was returned by [_1](ID: [_2])',
      $author->nickname, $author->id
    );
  } elsif ( $status == APPROVED ) {
    $id = 'approved_email';
    $header->{Subject} = _t(
      'The application for approval was approved by [_1](ID: [_2])',
      $author->nickname, $author->id
    );
  }

  my $params = {
    id => $id,
    cr2_status => $status,
    type => $class,
  };
  $params->{edit_url} = check_url($app, $obj, 'view');
  $params->{preview_url} = check_url($app, $obj, 'preview');

  my $tmpl = MT->model('template')->load({ blog_id => 0, identifier => $id });
  my $ctx = $tmpl->context;
  $ctx->stash('author', $author);
  $ctx->stash('blog_id', $blog->id);
  $ctx->stash('blog', $blog);
  $ctx->stash('entry', $obj) if $class eq 'entry' || $class eq 'page';
  $ctx->stash('content', $obj);
  $ctx->stash('content_type', MT->model('content_type')->load({ id => $obj->content_type_id }))
    if $class =~ m!^\d+$!;
  $ctx->stash('cr2_comment', $comment);
  $ctx->stash('cr2_approval', $approval);
  my $html = $tmpl->output($params);
  unless ( $html ) {
    return MT::log({
      blog_id => 0,
      level => MT::Log::WARNING,
      message => _t('Failure to load a template(Type: [_1]) to send emails for workflow. [_2]', _t($id), $tmpl->errstr),
    });
  }

  my $obj_author = $obj->author;
  my ($users, @authors) = ([], ());
  foreach my $step ( @{ $steps } ) {
    push @$users, (map { $_ } @$step);
  }
  push @$users, $obj_author->id;
  @authors = MT->model('author')->load({id => $users});

  my $author_sent = 0;
  foreach my $user ( @authors ) {
    send_notice($user->email, $header, $html, $blog->id);
    ++$author_sent if $obj->author->email == $user->email;
  }
}

sub cf_meta {
  my ($app, $obj) = @_;
  my $q = $app->param;
  my $blog_id = $app->param('blog_id') || 0;

  return {} if !$q->param('customfield_beacon');

  require CustomFields::Field;
  my $meta = {};
  foreach ($q->param()) {
    next if $_ eq 'customfield_beacon';
    if (m/^customfield_(.*?)$/) {
      my $field_name = $1;
      if ( m/^customfield_(.+?)_cb_beacon$/ ) {
        $field_name = $1;
        $meta->{$field_name} = defined( $q->param("customfield_$field_name") )
          ? $q->param("customfield_$field_name")
          : '0';
      }
      else {
        my $field = CustomFields::Field->load(
          {
            $blog_id ? ( blog_id => [ $blog_id, 0 ] ) : ( blog_id => $blog_id ),
            basename => $field_name
          }
        ) or next;
        if ($field->type eq 'datetime') {
          $meta->{$field_name} =
            $q->param("customfield_$field_name") ne ''
            ? $q->param("customfield_$field_name")
            : undef;
        } else {
          $meta->{$field_name} = $q->param("customfield_$field_name");
        }
      }
    }
  }
  $meta;
}

sub send_notice {
  my ($to, $header, $body, $blog_id) = @_;
  # TODO: I should add some ways of notice in the future.
  send_email(@_);
}

sub send_email {
  my ($to, $header, $body, $blog_id) = @_;
  $header->{To} = $to;
  unless ( MT::Mail->send($header, $body) ) {
    MT::log({
      blog_id => defined $blog_id ? $blog_id : 0,
      level => MT::Log::INFO,
      message => _t('Failure to send a email. [_1]', MT::Mail->errstr),
      metadata => _t(q{SUBJECT:
[_1]

BODY:
[_2]}, $header->{Subject}, $body),
    });
  }
  1;
}

sub sanitize {
  MT::Util::sanitize_embed(@_);
}

sub get_config_value {
    my ($key, $scope) = @_;
    my $plugin = plugin();
    my $config = $plugin->get_config_hash( $scope ? $scope : _scope() );
    return $config->{$key} if exists($config->{$key});
    $plugin->get_config_value($key, 'system');
}

sub load_latest_revision {
    my ($app, $class, $id) = @_;
    my ($id_col);
    if ( $class eq 'entry' || $class eq 'page' ) {
      $id_col = 'entry_id';
    } else {
      $class = 'content_data' ;
      $id_col = 'cd_id';
    }
    my $pkg = $app->model("$class:revision");
    return undef unless $pkg;
    my $terms;
    $terms->{$id_col} = $id;
    my $args;
    $args->{limit} = 1;
    $args->{'sort'} = 'rev_number';
    $args->{direction} = 'descend';
    my @revs = $pkg->load($terms, $args);
    @revs ? shift @revs : undef;
}

sub serialize {
  MT::Serialize->serialize($_[0]);
}

sub unserialize {
  MT::Serialize->unserialize($_[0]);
}

sub register_future_que {
  my ($rev_obj, $ds) = @_;
  my $prefix = 'cr2_future_que';
  my $class = MT->instance->model('session');
  my $id_col = $ds .'_id';
  my $name = join(':', ($prefix, $ds, $rev_obj->$id_col, $rev_obj->id));
  my $sess = $class->load({ name => $name }) || $class->new;
  require MT::App;
  $sess->id( MT::App::make_magic_token() );
  $sess->kind('CR');
  $sess->start(time);
  $sess->duration( time + 365 * 86400 );
  $sess->name( $name );
  $sess->set( 'object_id', $rev_obj->$id_col );
  $sess->set( 'revision_id', $rev_obj->id );
  unless ( $sess->save ) {
    MT::log( _t('Failure to save a session of future post que. [_1]', $sess->errstr) );
  }
  $sess;
}

sub set_meta {
    my $plugin = MT->component("Commercial");
    my ( $app, $obj, $meta ) = @_;
    return 0 unless $obj;
    unless ( defined $meta ) {
      $meta = {};
      my %params = $app->param_hash;
      foreach ( keys %params ) {
        next unless m!^customfield_(.+)!;
        next if $1 eq 'beacon';
        $meta->{$1} = $params{'customfield_' .$1} || undef;
      }
    }
    my $obj_type
        = $obj->can('class_type') ? $obj->class_type : $obj->datasource;
    if ( $obj->has_meta ) {
        my $updated = 0;
        foreach my $mf ( keys %$meta ) {
            if ( $obj->has_meta( 'field.' . $mf ) ) {
                $obj->meta( 'field.' . $mf, $meta->{$mf} );
            }
        }
    }
    else {
        my $id = $obj->id;

        require MT::PluginData;
        my $meta_data = MT::PluginData->get_by_key(
            { plugin => 'CustomFields', key => "${obj_type}_${id}" } );
        $meta_data->data( { customfields => $meta } );
    }

    return 1;
}

sub now {
  my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);  
  sprintf("%04d%02d%02d%02d%02d%02d", $year + 1900, $mon + 1, $mday, $hour, $min, $sec);
}

sub gather_cb_beacon {
  my $app = shift;
  my %params = $app->param_hash;
  foreach ( keys %params ) {
    next unless m!^(customfield_.+?)_cb_beacon$!;
    $app->param($1, '') unless defined $params{$1};
  }
}

sub is_mtcloud {
  my $app = MT->instance;
  return 0 unless defined $app->{cfg};
  return 0 unless defined $app->{cfg}->{__dbvar};
  return 0 unless defined $app->{cfg}->{__dbvar}->{pluginschemaversion};
  return 0 unless defined $app->{cfg}->{__dbvar}->{pluginschemaversion}->{cloud};
  1;
}

sub add_alt_tmpl_path_mtcloud {
  return 0 unless is_mtcloud;
  MT->instance->config('AltTemplatePath', File::Spec->catdir('/data', 'file', 'plugins', plugin()->id, 'tmpl'));
}

1;